<?php
//  ------------------------------------------------------------------------ //
//                XOOPS - PHP Content Management System                      //
//                    Copyright (c) 2000 XOOPS.org                           //
//                       <http://www.xoops.org/>                             //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //
// Author: Kazumi Ono (AKA onokazu)                                          //
// URL: http://www.myweb.ne.jp/, http://www.xoops.org/, http://jp.xoops.org/ //
// Project: The XOOPS Project                                                //
// ------------------------------------------------------------------------- //
/**
 * Manage template files
 *
 * @copyright    http://www.impresscms.org/ The ImpressCMS Project
 * @license    LICENSE.txt
 */

namespace ImpressCMS\Core\Models;

use ImpressCMS\Core\Database\Criteria\CriteriaCompo;
use ImpressCMS\Core\Database\Criteria\CriteriaItem;

/**
 * Template file handler class.
 * This class is responsible for providing data access mechanisms to the data source
 * of template file class objects.
 *
 * @package    ICMS\View\Template\File
 * @author    Kazumi Ono <onokazu@xoops.org>
 * @copyright    Copyright (c) 2000 XOOPS.org
 */
class TemplateFileHandler extends AbstractExtendedHandler {
	private $_prefetch_cache = array();

	public function __construct(&$db) {
		parent::__construct($db, 'view_template_file', 'tpl_id', 'tpl_tplset', 'tpl_file', 'icms', 'tplfile', 'tpl_id');
	}

	/**
	 * Loads Template source from DataBase
	 *
	 * @see TemplateFile
	 * @param TemplateFile $tplfile object of the template file to load
	 * @return bool TRUE on success, FALSE if fail
	 */
	public function loadSource(TemplateFile &$tplfile) {
		if (!$tplfile->getVar('tpl_source')) {
			$sql = 'SELECT tpl_source FROM ' . $this->table
				. " WHERE tpl_id='" . $tplfile->getVar('tpl_id') . "'";
			if (!$result = $this->db->query($sql)) {
				return false;
			}
			$myrow = $this->db->fetchArray($result);
			$tplfile->assignVar('tpl_source', $myrow['tpl_source']);
		}
		return true;
	}

	/**
	 * forces Template source into the DataBase
	 * @param TemplateFile $tplfile object of the template file to load
	 * @return bool TRUE on success, FALSE if fail
	 */
	public function forceUpdate(&$tplfile) {
			if ($tplfile->isNew()) {
							return false;
			}
			return $tplfile->store(true);
	}

	/**
	 * Count some tplfiles for a module
	 *
	 * @param   string $tplset Template Set
	 * @return  array $ret containing number of templates in the tpl_set or empty array if fails
	 */
	public function getModuleTplCount($tplset) {
		$ret = array();
		$sql = 'SELECT tpl_module, COUNT(tpl_id) AS count FROM ' . $this->db->prefix('tplfile')
			. " WHERE tpl_tplset='" . $tplset . "' GROUP BY tpl_module";
		$result = $this->db->query($sql);
		if (!$result) {
			return $ret;
		}
		while ($myrow = $this->db->fetchArray($result)) {
			if ($myrow['tpl_module']) {
				$ret[$myrow['tpl_module']] = $myrow['count'];
			}
		}
		return $ret;
	}

	/**
	 * find tplfiles matching criteria
	 *
	 * @param   string  $tplset             template set
	 * @param   string  $type               template type
	 * @param   int     $refid              ref id
	 * @param   string  $module             module
	 * @param   string  $file               template file
	 * @param   bool    $getsource = false  get source or not
	 * @return  array $ret containing number of templates in the tpl_set or empty array if fails
	 */
	public function find($tplset = null, $type = null, $refid = null, $module = null, $file = null, $getsource = false) {
		$criteria = new CriteriaCompo();
		if (isset($tplset)) {
			$criteria->add(new CriteriaItem('tpl_tplset', $tplset));
		}
		if (isset($module)) {
			$criteria->add(new CriteriaItem('tpl_module', $module));
		}
		if (isset($refid)) {
			$criteria->add(new CriteriaItem('tpl_refid', $refid));
		}
		if (isset($file)) {
			$criteria->add(new CriteriaItem('tpl_file', $file));
		}
		if (isset($type)) {
			if (is_array($type)) {
				$criteria2 = new CriteriaCompo();
				foreach ($type as $t) {
					$criteria2->add(new CriteriaItem('tpl_type', $t), 'OR');
				}
				$criteria->add($criteria2);
			} else {
				$criteria->add(new CriteriaItem('tpl_type', $type));
			}
		}
		return $this->getObjects($criteria, false, true);
	}

	/**
	 * Does the template exist in the database in the template set
	 *
	 * @param   string  $tplname        template name
	 * @param   string  $tplset_name    template set name
	 * @return  bool true if exists, false if not
	 */
	public function templateExists($tplname, $tplset_name) {
		$criteria = new CriteriaCompo(new CriteriaItem('tpl_file', trim($tplname)));
		$criteria->add(new CriteriaItem('tpl_tplset', trim($tplset_name)));
		if ($this->getCount($criteria) > 0) {
			return true;
		}
		return false;
	}

	/**
	 * Prefetch blocks to reduce the amount of queries required by Smarty to generate all blocks
	 * This function is called exclusively in PageBuilder
	 *
	 * @global	array	$icmsConfig		icms configuration array
	 * @param	array	$block_arr		array of blocks to prefetch
	 * @return	bool					false if there are no blocks to prefetch, otherwise true
	 */
	public function prefetchBlocks(&$block_arr) {
		global $icmsConfig;

		if (count($block_arr) === 0) {
			return false;
		}
		$tplNames = array();
		foreach ($block_arr as $block) {
			$tplNames[] = $block->template?: 'system_block_dummy.html';
		}

		$criteria = new CriteriaCompo(
			new CriteriaItem(
				'tpl_tplset',
				'(' .
				implode(
					',',
					array_map(
						[$this->db, 'quoteString'],
						array_unique(
							[
								$icmsConfig['template_set'],
								'default'
							]
						)
					)
				) .
				')',
				'IN'
			)
		);
		$criteria->add(
			new CriteriaItem(
				'tpl_file',
				'(' .
				implode(
					',',
					array_map(
						[$this->db, 'quoteString'],
						array_unique(
							$tplNames
						)
					)
				) .
				')',
				'IN'
			)
		);
		$criteria->setSort('tpl_refid');

		foreach ($this->getObjects($criteria) as $tpl) {
			$this->_prefetch_cache[$tpl->tpl_id] = $tpl;
		}

		return true;
	}

	/**
	 * Return a prefetched block. This function only works if prefetchBlocks was called in advance.
	 * This function is used in the user function smarty_resource_db_tplinfo().
	 *
	 * @param	string		$tplset		template set that's currently in use
	 * @param	string		$tpl_name	name of the template
	 * @return	TemplateFile[]			array of templates (just one item)
	 */
	public function getPrefetchedBlock($tplset, $tpl_name) {
		foreach ($this->_prefetch_cache as $block) {
			if ($block->tpl_tplset == $tplset && $block->tpl_file == $tpl_name) {
				return array($block);
			}
		}

		/**
		 * try to get the template from the default template set (we've also prefetched it) if the
		 * template set is different from default
		 */
		if ($tplset != 'default') {
			foreach ($this->_prefetch_cache as $block) {
				if ($block->tpl_tplset == "default" && $block->tpl_file == $tpl_name) {
					return array($block);
				}
			}
		}

		/**
		 * In case nothing was found, the following fallback tries to read the template again.
		 * This is the case for all non-block templates since blocks are prefetched before the
		 * content template(s) are required.
		 * To avoid further queries for the same block, we're adding it to the cache
		 */
		$blocks = $this->find($tplset, null, null, null, $tpl_name, true);
		$this->_prefetch_cache = array_merge($this->_prefetch_cache, $blocks);
				return $blocks;
	}
}